#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define GNAME_MAX 100

const char FMT[] = "%s_%%%s%zu.%zuf";
char FORMAT[GNAME_MAX];

/*!
 * \brief Get number of decimals from a string
 *
 * \param str String to analyze
 * 
 * \return number of decimals
 */
size_t get_numb_of_decimals(const char *str)
{
    int sep = '.';
    size_t len;
    char *sep_ptr = strchr(str, sep);
    if (sep_ptr == NULL)
        return 0;
    len = strlen(str);
    return len - (size_t)(sep_ptr - str) - 1;
}

/*!
 * \brief Generate the format string
 *
 * \param basename String with the basename
 * \param digits Number of digits number
 * \param decimals Number of decimal number
 * \param filler String used to fill, default is 0
 * 
 * \return Format string
 */
char *get_format_name(const char *basename, size_t digits, size_t decimals, 
                      const char *filler)
{
    size_t point = (decimals)? 1 : 0;
    if (filler == NULL)
        filler = "0";
    sprintf(FORMAT, FMT, basename, filler, digits + decimals + point, decimals);
    return FORMAT;
}

/*!
 * \brief Test functions
 */
int main(){
    int i, j, f;
    const char *decimals_strings[4] = {"12345.6789", "12345.67", 
                                       "12345.", "12345"};
    size_t digits[] = {2, 3, 4};
    size_t decimals[] = {0, 1, 2, 3, 4};
    const char basename[] = "basename";
    double number = 12.34567890;
    const char *fillers[4] = {"0", ""};
    char result[GNAME_MAX];
    char *fmt;

    printf("\n\nTest get_numb_of_decimals\n");
    for (i = 0; i < 4; i++){
        printf("Number of decimals in %s => %zu\n", 
               decimals_strings[i], get_numb_of_decimals(decimals_strings[i]));
    }

    printf("\n\nTest get_format_name\n");
    for (f = 0; f < 2; f++){
        for (i = 0; i < 3; i++){
            for (j = 0; j < 5; j++){
                fmt = get_format_name(basename, digits[i], decimals[j], fillers[f]);
                sprintf(result, fmt, number);
                printf("basename: %s, digits: %zu, decimals: %zu, filler: %s => %s => %s\n",
                       basename, digits[i], decimals[j], fillers[f], fmt, result);
            }
        }
    }

    return 0;
}
